@@ -1,3 +1,5 @@
+package org.wikidata.wdtk.wikibaseapi;
+
 /*
  * #%L
  * Wikidata Toolkit Wikibase API
@@ -18,17 +20,13 @@
  * #L%
  */
 
-package org.wikidata.wdtk.wikibaseapi;
-
-import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertNull;
-import static org.junit.jupiter.api.Assertions.assertThrows;
-import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.only;
+import static org.mockito.Mockito.when;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.verifyNoInteractions;
 import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeStringValue;
+import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeQuantityValue;
 import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeWikidataFormIdValue;
 import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeWikidataItemIdValue;
 import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeWikidataLexemeIdValue;
@@ -36,14 +34,15 @@ import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeWikidataProperty
 import static org.wikidata.wdtk.datamodel.helpers.Datamodel.makeWikidataSenseIdValue;
 
 import java.io.IOException;
+import java.math.BigDecimal;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import org.junit.jupiter.api.BeforeEach;
-import org.junit.jupiter.api.Test;
+import org.junit.Before;
+import org.junit.Test;
 import org.wikidata.wdtk.datamodel.helpers.AliasUpdateBuilder;
 import org.wikidata.wdtk.datamodel.helpers.Datamodel;
 import org.wikidata.wdtk.datamodel.helpers.FormUpdateBuilder;
@@ -59,23 +58,32 @@ import org.wikidata.wdtk.datamodel.helpers.TermUpdateBuilder;
 import org.wikidata.wdtk.datamodel.interfaces.DatatypeIdValue;
 import org.wikidata.wdtk.datamodel.interfaces.EntityDocument;
 import org.wikidata.wdtk.datamodel.interfaces.EntityUpdate;
+import org.wikidata.wdtk.datamodel.interfaces.FormDocument;
 import org.wikidata.wdtk.datamodel.interfaces.FormUpdate;
 import org.wikidata.wdtk.datamodel.interfaces.ItemDocument;
 import org.wikidata.wdtk.datamodel.interfaces.ItemIdValue;
 import org.wikidata.wdtk.datamodel.interfaces.ItemUpdate;
+import org.wikidata.wdtk.datamodel.interfaces.LexemeDocument;
 import org.wikidata.wdtk.datamodel.interfaces.LexemeUpdate;
 import org.wikidata.wdtk.datamodel.interfaces.MediaInfoDocument;
 import org.wikidata.wdtk.datamodel.interfaces.MediaInfoIdValue;
 import org.wikidata.wdtk.datamodel.interfaces.MonolingualTextValue;
 import org.wikidata.wdtk.datamodel.interfaces.PropertyDocument;
 import org.wikidata.wdtk.datamodel.interfaces.PropertyIdValue;
+import org.wikidata.wdtk.datamodel.interfaces.SenseDocument;
 import org.wikidata.wdtk.datamodel.interfaces.SenseUpdate;
 import org.wikidata.wdtk.datamodel.interfaces.Statement;
+import org.wikidata.wdtk.testing.MockStringContentFactory;
 import org.wikidata.wdtk.util.CompressionType;
 import org.wikidata.wdtk.wikibaseapi.apierrors.MediaWikiApiErrorException;
 import org.wikidata.wdtk.wikibaseapi.apierrors.TagsApplyNotAllowedException;
 import org.wikidata.wdtk.wikibaseapi.apierrors.TokenErrorException;
 
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.ObjectMapper;
+
+import static org.junit.Assert.*;
+
 public class WikibaseDataEditorTest {
 
 	MockBasicApiConnection con;
@@ -83,8 +91,9 @@ public class WikibaseDataEditorTest {
 	PropertyIdValue P31 = Datamodel.makeWikidataPropertyIdValue("P31");
 	static final String TEST_GUID = "427C0317-BA8C-95B0-16C8-1A1B5FAC1081";
 	MockGuidGenerator guids = new MockGuidGenerator(TEST_GUID);
+	ObjectMapper mapper = new ObjectMapper();
 
-	@BeforeEach
+	@Before
 	public void setUp() throws IOException {
 		this.con = new MockBasicApiConnection();
 		Map<String, String> params = new HashMap<>();
@@ -225,7 +234,7 @@ public class WikibaseDataEditorTest {
 		assertEquals(-1, wde.getRemainingEdits());
 	}
 
-	@Test
+	@Test(expected = TokenErrorException.class)
 	public void testCreateItemBadToken() throws IOException,
 			MediaWikiApiErrorException {
 		WikibaseDataEditor wde = new WikibaseDataEditor(this.con,
@@ -246,7 +255,7 @@ public class WikibaseDataEditorTest {
 		this.con.setWebResourceFromPath(params, this.getClass(),
 				"/error-badtoken.json", CompressionType.NONE);
 
-		assertThrows(TokenErrorException.class, () -> wde.createItemDocument(itemDocument, "My summary", null));
+		wde.createItemDocument(itemDocument, "My summary", null);
 	}
 
 	@Test
@@ -317,7 +326,7 @@ public class WikibaseDataEditorTest {
 		assertEquals(expectedResultDocument, result);
 	}
 
-	@Test
+	@Test(expected = IOException.class)
 	public void testCreateItemBadEntityDocumentJson() throws IOException,
 			MediaWikiApiErrorException {
 		// Test what happens if the API returns JSON without an actual entity
@@ -340,10 +349,10 @@ public class WikibaseDataEditorTest {
 		params.put("data", data);
 		con.setWebResource(params, expectedResult);
 
-		assertThrows(IOException.class, () -> wde.createItemDocument(itemDocument, "My summary", null));
+		wde.createItemDocument(itemDocument, "My summary", null);
 	}
 
-	@Test
+	@Test(expected = IOException.class)
 	public void testCreateItemMissingEntityDocumentJson() throws IOException,
 			MediaWikiApiErrorException {
 		WikibaseDataEditor wde = new WikibaseDataEditor(this.con,
@@ -364,7 +373,7 @@ public class WikibaseDataEditorTest {
 		params.put("data", data);
 		con.setWebResource(params, expectedResult);
 
-		assertThrows(IOException.class, () -> wde.createItemDocument(itemDocument, "My summary", null));
+		wde.createItemDocument(itemDocument, "My summary", null);
 	}
 
 	@Test
@@ -841,9 +850,9 @@ public class WikibaseDataEditorTest {
 		assertEquals(expectedResultDocument, result);
 	}
 	
-	@Test
 	@Deprecated
 	@SuppressWarnings("deprecation")
+	@Test(expected = TagsApplyNotAllowedException.class)
 	public void testApplyInvalidTag() throws MediaWikiApiErrorException, IOException {
 		WikibaseDataEditor wde = new WikibaseDataEditor(this.con,
 				Datamodel.SITE_WIKIDATA);
@@ -874,32 +883,47 @@ public class WikibaseDataEditorTest {
 				+ "\"servedby\":\"mw1276\"}";
 		con.setWebResource(params, expectedResult);
 
-		assertThrows(TagsApplyNotAllowedException.class, () -> wde.updateTermsStatements(itemDocument, Collections.<MonolingualTextValue>emptyList(),
+		wde.updateTermsStatements(itemDocument, Collections.<MonolingualTextValue>emptyList(),
 				Collections.singletonList(description),	Collections.<MonolingualTextValue>emptyList(),
 				Collections.<MonolingualTextValue>emptyList(), Collections.<Statement>emptyList(),
-				Collections.<Statement>emptyList(), "testing tags", Collections.singletonList("tag_which_does_not_exist")));
+				Collections.<Statement>emptyList(), "testing tags", Collections.singletonList("tag_which_does_not_exist"));
 	}
 
-	private WbEditingAction mockEntityUpdate(EntityUpdate update) throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mock(WbEditingAction.class);
+	private EditingResult mockEntityUpdate(WbEditingAction action, EntityUpdate update) throws MediaWikiApiErrorException, IOException {
 		WikibaseDataFetcher fetcher = new WikibaseDataFetcher(con, Datamodel.SITE_WIKIDATA);
 		WikibaseDataEditor wde = new WikibaseDataEditor(action, fetcher, Datamodel.SITE_WIKIDATA, guids);
-		wde.editEntityDocument(update, false, "test summary", Arrays.asList("tag1"));
-		return action;
+		return wde.editEntityDocument(update, false, "test summary", Arrays.asList("tag1"));
+	}
+	
+	private JsonNode json(String resourceFileName) {
+	    try {
+    	    String contents = MockStringContentFactory
+                    .getStringFromUrl(WikibaseDataEditor.class.getResource(resourceFileName));
+    	    return mapper.readTree(contents);
+	    } catch (Exception e) {
+	        throw new IllegalArgumentException("Unable to read test JSON resource "+resourceFileName);
+	    }
 	}
 
 	@Test
 	public void testReductionToSetNewClaim() throws MediaWikiApiErrorException, IOException {
 		ItemIdValue subject = makeWikidataItemIdValue("Q1");
 		Statement statement = StatementBuilder.forSubjectAndProperty(subject, makeWikidataPropertyIdValue("P1"))
-				.withValue(makeStringValue("some value"))
+				.withValue(makeQuantityValue(new BigDecimal("456")))
 				.build();
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder.forBaseRevisionId(subject, 123)
+	    Statement statementWithId = statement.withStatementId(guids.freshStatementId(subject.getId()));
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetClaim(
+                JsonSerializer.getJsonString(statementWithId), false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetclaim.json"));
+		
+		EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder.forBaseRevisionId(subject, 123)
 				.updateStatements(StatementUpdateBuilder.create()
 						.add(statement)
 						.build())
 				.build());
-		Statement statementWithId = statement.withStatementId(guids.freshStatementId(subject.getId()));
+		
+		assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetClaim(
 				JsonSerializer.getJsonString(statementWithId), false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -908,12 +932,19 @@ public class WikibaseDataEditorTest {
 	public void testReductionToSetExistingClaim() throws MediaWikiApiErrorException, IOException {
 		ItemIdValue subject = makeWikidataItemIdValue("Q1");
 		Statement statement = StatementBuilder.forSubjectAndProperty(subject, makeWikidataPropertyIdValue("P1"))
-				.withValue(makeStringValue("some value"))
+				.withValue(makeQuantityValue(new BigDecimal("456")))
 				.withId(guids.freshStatementId(subject.getId()))
 				.build();
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder.forBaseRevisionId(subject, 123)
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetClaim(
+                JsonSerializer.getJsonString(statement), false, 123, "test summary", Arrays.asList("tag1")))
+            .thenReturn(json("/wbsetclaim.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder.forBaseRevisionId(subject, 123)
 				.updateStatements(StatementUpdateBuilder.create().replace(statement).build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetClaim(
 				JsonSerializer.getJsonString(statement), false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -922,65 +953,106 @@ public class WikibaseDataEditorTest {
 	public void testReductionToRemoveClaims() throws MediaWikiApiErrorException, IOException {
 		ItemIdValue subject = makeWikidataItemIdValue("Q1");
 		String id = guids.freshStatementId(subject.getId());
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder.forBaseRevisionId(subject, 123)
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbRemoveClaims(Arrays.asList(id), false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbremoveclaims.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder.forBaseRevisionId(subject, 123)
 				.updateStatements(StatementUpdateBuilder.create().remove(id).build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbRemoveClaims(Arrays.asList(id), false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToSetLabel() throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetLabel(
+                "Q1", null, null, null, "en", "hello", false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetlabel.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateLabels(TermUpdateBuilder.create()
 						.put(Datamodel.makeMonolingualTextValue("hello", "en"))
 						.build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetLabel(
 				"Q1", null, null, null, "en", "hello", false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToSetNullLabel() throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetLabel(
+                "Q1", null, null, null, "en", null, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetlabel-null.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateLabels(TermUpdateBuilder.create().remove("en").build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetLabel(
 				"Q1", null, null, null, "en", null, false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToSetDescription() throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetDescription(
+                "Q1", null, null, null, "en", "hello", false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetdescription.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateDescriptions(TermUpdateBuilder.create()
 						.put(Datamodel.makeMonolingualTextValue("hello", "en"))
 						.build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetDescription(
 				"Q1", null, null, null, "en", "hello", false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToSetNullDescription() throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetDescription(
+                "Q1", null, null, null, "en", null, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetdescription-null.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateDescriptions(TermUpdateBuilder.create().remove("en").build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetDescription(
 				"Q1", null, null, null, "en", null, false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToSetAliases() throws MediaWikiApiErrorException, IOException {
-		WbEditingAction action = mockEntityUpdate(ItemUpdateBuilder
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbSetAliases("Q1", null, null, null, "en",
+                Arrays.asList("hello"), Arrays.asList("bye"), null, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(json("/wbsetaliases-add-remove.json"));
+		
+        EditingResult result = mockEntityUpdate(action, ItemUpdateBuilder
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateAliases("en", AliasUpdateBuilder.create()
 						.add(Datamodel.makeMonolingualTextValue("hello", "en"))
 						.remove(Datamodel.makeMonolingualTextValue("bye", "en"))
 						.build())
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbSetAliases("Q1", null, null, null, "en",
 				Arrays.asList("hello"), Arrays.asList("bye"), null, false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -991,10 +1063,19 @@ public class WikibaseDataEditorTest {
 				.forEntityId(makeWikidataSenseIdValue("L1-S1"))
 				.updateGlosses(TermUpdateBuilder.create().remove("en").build())
 				.build();
-		WbEditingAction action = mockEntityUpdate(LexemeUpdateBuilder
+		SenseDocument senseDocument = mock(SenseDocument.class);
+		when(senseDocument.getRevisionId()).thenReturn(1234L);
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbEditEntity("L1-S1", null, null, null, JsonSerializer.getJsonString(update),
+                false, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(senseDocument);
+		
+        EditingResult result = mockEntityUpdate(action, LexemeUpdateBuilder
 				.forBaseRevisionId(makeWikidataLexemeIdValue("L1"), 123)
 				.updateSense(update)
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbEditEntity("L1-S1", null, null, null, JsonSerializer.getJsonString(update),
 				false, false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -1005,10 +1086,19 @@ public class WikibaseDataEditorTest {
 				.forEntityId(makeWikidataFormIdValue("L1-F1"))
 				.updateRepresentations(TermUpdateBuilder.create().remove("en").build())
 				.build();
-		WbEditingAction action = mockEntityUpdate(LexemeUpdateBuilder
+		FormDocument formDocument = mock(FormDocument.class);
+        when(formDocument.getRevisionId()).thenReturn(1234L);
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbEditEntity("L1-F1", null, null, null, JsonSerializer.getJsonString(update),
+                false, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(formDocument);
+		
+        EditingResult result = mockEntityUpdate(action, LexemeUpdateBuilder
 				.forBaseRevisionId(makeWikidataLexemeIdValue("L1"), 123)
 				.updateForm(update)
 				.build());
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbEditEntity("L1-F1", null, null, null, JsonSerializer.getJsonString(update),
 				false, false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -1019,7 +1109,16 @@ public class WikibaseDataEditorTest {
 				.forBaseRevisionId(makeWikidataLexemeIdValue("L1"), 123)
 				.setLanguage(Datamodel.makeWikidataItemIdValue("Q1"))
 				.build();
-		WbEditingAction action = mockEntityUpdate(update);
+		LexemeDocument lexemeDocument = mock(LexemeDocument.class);
+		when(lexemeDocument.getRevisionId()).thenReturn(1234L);
+		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbEditEntity("L1", null, null, null, JsonSerializer.getJsonString(update),
+                false, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(lexemeDocument);
+		
+        EditingResult result = mockEntityUpdate(action, update);
+        
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbEditEntity("L1", null, null, null, JsonSerializer.getJsonString(update),
 				false, false, 123, "test summary", Arrays.asList("tag1"));
 	}
@@ -1030,19 +1129,31 @@ public class WikibaseDataEditorTest {
 				.forBaseRevisionId(makeWikidataItemIdValue("Q1"), 123)
 				.updateLabels(TermUpdateBuilder.create().remove("en").build())
 				.build();
+		ItemDocument itemDocument = mock(ItemDocument.class);
+		when(itemDocument.getRevisionId()).thenReturn(1234L);
 		WbEditingAction action = mock(WbEditingAction.class);
+		when(action.wbEditEntity("Q1", null, null, null, JsonSerializer.getJsonString(update),
+                true, false, 123, "test summary", Arrays.asList("tag1")))
+		    .thenReturn(itemDocument);
 		WikibaseDataFetcher fetcher = new WikibaseDataFetcher(con, Datamodel.SITE_WIKIDATA);
 		WikibaseDataEditor wde = new WikibaseDataEditor(action, fetcher, Datamodel.SITE_WIKIDATA, guids);
-		wde.editEntityDocument(update, true, "test summary", Arrays.asList("tag1"));
+		
+		EditingResult result = wde.editEntityDocument(update, true, "test summary", Arrays.asList("tag1"));
+		
+        assertEquals(result, new EditingResult(1234L));
 		verify(action, only()).wbEditEntity("Q1", null, null, null, JsonSerializer.getJsonString(update),
 				true, false, 123, "test summary", Arrays.asList("tag1"));
 	}
 
 	@Test
 	public void testReductionToNoEdit() throws MediaWikiApiErrorException, IOException {
-		verifyNoInteractions(mockEntityUpdate(LexemeUpdateBuilder
-				.forBaseRevisionId(makeWikidataLexemeIdValue("L1"), 123)
-				.build()));
+	    WbEditingAction action = mock(WbEditingAction.class);
+	    EditingResult result = mockEntityUpdate(action, LexemeUpdateBuilder
+                .forBaseRevisionId(makeWikidataLexemeIdValue("L1"), 123)
+                .build());
+	    
+	    assertEquals(result, new EditingResult(0L));
+		verifyNoInteractions(action);
 	}
 
 }
